/*-
 * Copyright (c) 2010  Marin Atanasov Nikolov  <dnaeon@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _CONFIG_H_
#define _CONFIG_H_

#ifndef _SYS_QUEUE_H_
#include <sys/queue.h>
#endif

#define CONFIG_DEFAULT_FILE		     "/usr/local/etc/pkg_add_it.conf"

#define CONFIG_COMMENT			     '#'
#define CONFIG_NEWLINE			     '\n'
#define CONFIG_DELIMETER		     "="

#define CONFIG_EXPAND_DELIMITER		     '$'

#define CONFIG_FETCH_CMD		     "FETCH_CMD"
#define CONFIG_FETCH_CMD_ARGS		     "FETCH_CMD_ARGS"

#define CONFIG_INDEX_FILE		     "INDEX_FILE"

#define CONFIG_PKG_DIRS			     "PKG_DIRS"
#define CONFIG_PKG_URL			     "PKG_URL"
#define CONFIG_PKG_ROOT			     "PKG_ROOT"
#define CONFIG_PKG_ARCH			     "PKG_ARCH"
#define CONFIG_PKG_RELEASE		     "PKG_RELEASE"

#define CONFIG_PKG_INSTALL_CMD		     "PKG_INSTALL_CMD"
#define CONFIG_PKG_INSTALL_CMD_ARGS	     "PKG_INSTALL_CMD_ARGS"
#define CONFIG_PKG_INSTALL_CMD_REMOTE	     "PKG_INSTALL_CMD_REMOTE"
#define CONFIG_PKG_INSTALL_CMD_REMOTE_ARGS   "PKG_INSTALL_CMD_REMOTE_ARGS"

#define CONFIG_PKG_INFO_CMD		     "PKG_INFO_CMD"
#define CONFIG_PKG_INFO_CMD_ARGS	     "PKG_INFO_CMD_ARGS"

#define CONFIG_DEPS_FULL_TREE		     "DEPS_FULL_TREE"

#define CONFIG_SHOW_PORT_DIR		     "SHOW_PORT_DIR"
#define CONFIG_SHOW_INST_PREFIX		     "SHOW_INST_PREFIX"
#define CONFIG_SHOW_DESC		     "SHOW_DESC"
#define CONFIG_SHOW_DESC_FILE		     "SHOW_DESC_FILE"
#define CONFIG_SHOW_MAINT		     "SHOW_MAINT"
#define CONFIG_SHOW_CAT			     "SHOW_CAT"
#define CONFIG_SHOW_BDEPS		     "SHOW_BDEPS"
#define CONFIG_SHOW_RDEPS		     "SHOW_RDEPS"
#define CONFIG_SHOW_WWW			     "SHOW_WWW"
#define CONFIG_SHOW_EDEPS		     "SHOW_EDEPS"
#define CONFIG_SHOW_PDEPS		     "SHOW_PDEPS"
#define CONFIG_SHOW_FDEPS		     "SHOW_FDEPS"

typedef struct config {
   char	    *option;	/* option name  */
   char	    *value;	/* option value */
   unsigned line;	/* option line in configuration file */
   char	    required;   /* is option required? */
   char     is_bool;	/* 1 for a boolean optin, 0 for a string */

   STAILQ_ENTRY(config) entries; 
} CONFIG;

int   config_check_options(void);
int   config_expand_options(void);
void  config_init();
void  config_finish(void);
void  config_register_option(const char *option, char required, char is_bool);
char *config_get_option(const char *option);
int   config_get_bool_option(const char *option);
void  config_read_file(const char *filename);
void  config_set_option(const char *option, const char *value, unsigned int line);
void  config_set_value(const char *option, const char *newvalue);
char *trim_spaces(char *str);
char *str_replace (char *str, char *old, char *new);

#endif /* !_CONFIG_H_ */
